(document:surround "/std/frame")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-x11")

;;;;;;;;;;;;;;;;;;;;;; current data
(define current-data (woo-read-first "/x11"))

(define auto-cardname (woo-get-option current-data 'auto_cardname))
(define auto-monitor (woo-get-option current-data 'auto_monitor))

(define current-xdriver-name (woo-get-option current-data 'xdrivername #f))
(define current-monitor (woo-get-option current-data 'monitor #f))
(define current-xdepth (woo-get-option current-data 'xdepth #f))
(define current-xresolution (woo-get-option current-data 'xresolution #f))
(define current-onboot (woo-get-option current-data 'onboot #f))

;;;;;;;;;;;;;;;;;;;;;;
(define module (_ "Display properties"))

(define monitor-list (woo-list/name+label "/x11/avail_model"))
(define xdepth-list (woo-list/name+label "/x11/avail_xdepth"))
(define xresolution-list (woo-list/name+label "/x11/avail_xresolution"))

(define (get-variant lst w)
  (car (list-ref lst (w current))))

(define (get-xresolution)
  (let ((num (resolution current)))
    (if (= num 0)
        "auto"
        (string-join (reverse (map car (list-tail xresolution-list num)))
                     ","))))

(define (current-index x lst)
  (and (string? x)
       (list-index (lambda(y) (string=? (car y) x)) lst)))

(define (default-variant lst cur def)
  (list rows (map cdr lst)
        current (or (current-index  cur lst) def)))

(define (apply-data url really)
  (woo-write url 'commit really
             'monitor (get-variant monitor-list monitor)
             'xdepth (get-variant xdepth-list depth)
             'xresolution (get-xresolution)
             'onboot (onboot state)))

(define (test-not-apply)
  (woo-catch/message
   (thunk (apply-data "/x11" #f)
          (woo-write "/x11/test"))))

;; save data if url changed
(define (change url)
  (apply-data "/x11" #f)
  (frame:replace url))

;;;;;;;;;;;;;;;;;;;;;;;;; ui

(gridbox columns "20;15;35;20"
	           ;;0
		   (spacer)
		   (label (_ "Detected hardware:"))
		   (spacer)
		   (spacer)
	           ;;1	   
                   (spacer) (label (bold (_ "Video card"))) (label auto-cardname) (spacer)
                   (spacer) (label (bold (_ "Monitor"))) (label auto-monitor) (spacer)
		   (spacer) (separator) (separator) (spacer)
		   ;;2
		   (spacer)
		   (label (_ "Current settings:"))
		   (spacer)
		   (spacer)
                   ;;3
                   (spacer)
                   (label (bold (_ "Monitor")))
		   (hbox
                     (document:id monitor (combobox (default-variant monitor-list current-monitor 0)))
                     (document:id ch-mon (button (_ "Choose other vendor"))))
		   (spacer)
                   ;;4
                   (spacer)
                   (label (bold (_ "Driver")))
                   (hbox
                    (document:id driver (label current-xdriver-name))
                    (document:id ch-drv (button (_ "Choose other driver"))))
                   (spacer)
                   ;;5
                   (spacer)
                   (label (bold (_ "Color depth")))
                   (document:id depth (combobox (default-variant xdepth-list current-xdepth 2)))
                   (spacer)
                   ;;6
                   (spacer)
                   (label (bold (_ "Resolution")))
                   (document:id resolution (combobox (default-variant xresolution-list current-xresolution 0)))
                   (spacer)
		   ;;7
		   (spacer)
		   (separator)
		   (spacer)
		   (spacer)
		   ;;8
		   (spacer)
		   (document:id onboot (checkbox (_ "Start graphical interface on system booting")))
		   (spacer)
		   (spacer))

;;;;;;;;;;;;;;;;;;; buttons
(hbox
 align "center"
 (or (global 'frame:next-button)
     (button (_ "Exit") (when clicked (document:end))))
 (button (_ "Test") (when clicked (test-not-apply)))
 (or (global 'frame:next-button)
     (button (_ "Save") (when clicked (apply-data "/x11" #t)))))


;;;;;;;;;;;;;;;;;;; actions
(ch-mon (when clicked (change "/x11/monitors")))
(ch-drv (when clicked (change "/x11/drivers")))

(onboot state (or current-onboot (->bool (global 'frame:next-button))))

;;;;;;;;;;;;;;;;;;; installer specific
(frame:on-next (thunk
                (or (and (apply-data "/x11" #t) (apply-data "/autoinstall/x11" #t))
                    'cancel)))
